# -*- coding: utf-8 -*-

import re
import os
from django.conf import settings
from django.shortcuts import redirect
from django.db.models import Q
from datetime import datetime, timedelta
from djobberbase.conf import settings as djobberbase_settings

def normalize_query(query_string,
                    findterms=re.compile(r'"([^"]+)"|(\S+)').findall,
                    normspace=re.compile(r'\s{2,}').sub):
    ''' Splits the query string in invidual keywords, getting rid of unecessary spaces
        and grouping quoted words together.
        Example:
        
        >>> normalize_query('  some random  words "with   quotes  " and   spaces')
        ['some', 'random', 'words', 'with quotes', 'and', 'spaces']
    
    '''
    return [normspace(' ', (t[0] or t[1]).strip()) for t in findterms(query_string)] 

def get_query(query_string, search_fields):
    ''' Returns a query, that is a combination of Q objects. That combination
        aims to search keywords within a model by testing the given search fields.
        It has been slightly modified from the original to support related fields.    
    '''    
    query = None # Query to search for every search term        
    terms = normalize_query(query_string)
    for term in terms:
        or_query = None # Query to search for a given term in each field
        for field_name in search_fields:
            if field_name == 'category' or field_name == 'jobtype' or field_name == 'city':
                q = Q(**{"%s__name__icontains" % field_name: term})
            else:
                q = Q(**{"%s__icontains" % field_name: term})
            if or_query is None:
                or_query = q
            else:
                or_query = or_query | q
        if query is None:
            query = or_query
        else:
            query = query & or_query
    return query

def handle_uploaded_file(f, name):
    file_uploads = djobberbase_settings.DJOBBERBASE_FILE_UPLOADS
    destination = open(file_uploads + name, 'wb+')
    for chunk in f.chunks():
        destination.write(chunk)
    destination.close()

def delete_uploaded_file(name):
    os.remove(name)

def minutes_between():
    minutes = djobberbase_settings.DJOBBERBASE_MINUTES_BETWEEN
    start = datetime.now() - timedelta(minutes=minutes)
    end = datetime.now()
    return (start, end)
    
def last_hour():
    start = datetime.now() - timedelta(hours=1)
    end = datetime.now()
    return (start, end)
	
def getIP(request):
    ip = request.META['REMOTE_ADDR']    
    if (not ip or ip == '127.0.0.1') and request.META.has_key('HTTP_X_FORWARDED_FOR'):
        ip = request.META['HTTP_X_FORWARDED_FOR']
    return ip



# -*- coding: utf-8 -*-

import os
import re
from django.conf import settings
from django.core.validators import email_re
from django.core.exceptions import ImproperlyConfigured

# General settings
DJOBBERBASE_SITE_NAME = getattr(settings, 'DJOBBERBASE_SITE_NAME', 'Djobberbase')
DJOBBERBASE_HTML_TITLE = getattr(settings, 'DJOBBERBASE_HTML_TITLE', 'Djobberbase :: A jobberBase clone written using the Django framework')
DJOBBERBASE_SITE_KEYWORDS = getattr(settings, 'DJOBBERBASE_SITE_KEYWORDS', ('job search', 'jobs', 'employment') )
DJOBBERBASE_SITE_DESCRIPTION = getattr(settings, 'DJOBBERBASE_SITE_DESCRIPTION', 'Djobberbase jobs.')
DJOBBERBASE_MINUTES_BETWEEN = getattr(settings, 'DJOBBERBASE_MINUTES_BETWEEN', 10)
DJOBBERBASE_MAX_UPLOAD_SIZE = getattr(settings, 'DJOBBERBASE_MAX_UPLOAD_SIZE', 3145728)
DJOBBERBASE_FILE_UPLOADS = getattr(settings, 'DJOBBERBASE_FILE_UPLOADS', './uploads/')
DJOBBERBASE_JOBS_PER_PAGE = getattr(settings, 'DJOBBERBASE_JOBS_PER_PAGE', 50)
DJOBBERBASE_JOBS_PER_SEARCH = getattr(settings, 'DJOBBERBASE_JOBS_PER_SEARCH', 25)
DJOBBERBASE_MAX_VISITS_PER_HOUR = getattr(settings, 'DJOBBERBASE_MAX_VISITS_PER_HOUR', 1)
DJOBBERBASE_CAPTCHA_POST = getattr(settings, 'DJOBBERBASE_CAPTCHA_POST', None)
DJOBBERBASE_CAPTCHA_APPLICATION = getattr(settings, 'DJOBBERBASE_CAPTCHA_APPLICATION', None)
DJOBBERBASE_CV_EXTENSIONS = getattr(settings, 'DJOBBERBASE_CV_EXTENSIONS', ('pdf', 'rtf', 'doc', 'docx', 'odt'))

# Custom URLs settings
def geturl(url_set, url, default):
    ''' It checks if the url is valid and not already in use, in case
        it's an invalid url then the default url is returned.
    '''
    u = getattr(settings, url, default)
    if re.match("^[a-zA-Z0-9_.-]+$", u) is not None and u not in url_set:
        url_set.append(u)
    else:
        raise ImproperlyConfigured('The url "%s" is already in use' % u)
    return u

url_set = []
DJOBBERBASE_POST_URL = geturl(url_set, 'DJOBBERBASE_POST_URL', 'post')
DJOBBERBASE_VERIFY_URL = geturl(url_set, 'DJOBBERBASE_VERIFY_URL', 'verify')
DJOBBERBASE_CONFIRM_URL = geturl(url_set, 'DJOBBERBASE_CONFIRM_URL', 'confirm')
DJOBBERBASE_JOB_URL = geturl(url_set, 'DJOBBERBASE_JOB_URL', 'job')
DJOBBERBASE_AT_URL = geturl(url_set, 'DJOBBERBASE_AT_URL', 'at')
DJOBBERBASE_JOBS_URL = geturl(url_set, 'DJOBBERBASE_JOBS_URL', 'jobs')
DJOBBERBASE_CITIES_URL = geturl(url_set, 'DJOBBERBASE_CITIES_URL', 'cities')
DJOBBERBASE_COMPANIES_URL = geturl(url_set, 'DJOBBERBASE_COMPANIES_URL', 'companies')
DJOBBERBASE_JOBS_IN_URL = geturl(url_set, 'DJOBBERBASE_JOBS_IN_URL', 'jobs-in')
DJOBBERBASE_JOBS_IN_OTHER_CITIES = geturl(url_set, 'DJOBBERBASE_JOBS_IN_OTHER_CITIES', 'jobs-in-other-cities')
DJOBBERBASE_JOBS_AT_URL = geturl(url_set, 'DJOBBERBASE_JOBS_AT_URL', 'jobs-at')
DJOBBERBASE_ACTIVATE_URL = geturl(url_set, 'DJOBBERBASE_ACTIVATE_URL', 'activate')
DJOBBERBASE_DEACTIVATE_URL = geturl(url_set, 'DJOBBERBASE_DEACTIVATE_URL', 'deactivate')
DJOBBERBASE_SEARCH_URL = geturl(url_set, 'DJOBBERBASE_SEARCH_URL', 'search')
DJOBBERBASE_UNAVAILABLE_URL = geturl(url_set, 'DJOBBERBASE_UNAVAILABLE_URL', 'job-unavailable')

# Mailing settings
DJOBBERBASE_ENABLE_NEW_POST_MODERATION = getattr(settings, 'DJOBBERBASE_ENABLE_NEW_POST_MODERATION', True)
DJOBBERBASE_ADMIN_EMAIL = getattr(settings, 'DJOBBERBASE_ADMIN_EMAIL', '')
DJOBBERBASE_ADMIN_NOTIFICATIONS = getattr(settings, 'DJOBBERBASE_ADMIN_NOTIFICATIONS', False)
DJOBBERBASE_POSTER_NOTIFICATIONS = getattr(settings, 'DJOBBERBASE_POSTER_NOTIFICATIONS', False)
DJOBBERBASE_APPLICATION_NOTIFICATIONS = getattr(settings, 'DJOBBERBASE_APPLICATION_NOTIFICATIONS', False)

DJOBBERBASE_NEW_POST_ADMIN_SUBJECT = getattr(settings, 
                                    'DJOBBERBASE_NEW_POST_ADMIN_SUBJECT', 
                                    '[ %(site_name)s  ] New job: %(job_title)s')

DJOBBERBASE_EDIT_POST_ADMIN_SUBJECT = getattr(settings, 
                                    'DJOBBERBASE_EDIT_POST_ADMIN_SUBJECT', 
                                    '[ %(site_name)s  ] Edited job: %(job_title)s')

DJOBBERBASE_MAIL_PENDING_SUBJECT = getattr(settings, 
                                    'DJOBBERBASE_MAIL_PENDING_SUBJECT', 
                                    'Your ad on %(site_name)s')

DJOBBERBASE_MAIL_PUBLISH_SUBJECT = getattr(settings, 
                                    'DJOBBERBASE_MAIL_PUBLISH_SUBJECT', 
                                    'Your ad on %(site_name)s was published')
 
DJOBBERBASE_MAIL_APPLY_ONLINE_SUBJECT = getattr(settings, 
                                    'DJOBBERBASE_MAIL_APPLY_ONLINE_SUBJECT', 
                                    '[ %(site_name)s ] I wish to apply for %(job_title)s')


# Markup settings
DJOBBERBASE_MARKUP_LANGUAGE = getattr(settings, 'DJOBBERBASE_MARKUP_LANGUAGE', None) #options: 'textile', 'markdown'




# -*- coding: utf-8 -*-

from django.core.mail import send_mail, EmailMessage
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.template import Context
from django.contrib.sites.models import Site
from djobberbase.helpers import getIP, handle_uploaded_file, delete_uploaded_file
from djobberbase.conf import settings as djobberbase_settings
from time import time
import threading

site_domain = Site.objects.get_current().domain

class MailPublishToAdmin(threading.Thread):

    def __init__(self, job, request):
        threading.Thread.__init__(self)
        plaintext = get_template('djobberbase/emails/publish_to_admin.txt')
        html = get_template('djobberbase/emails/publish_to_admin.html')
        template_vars = {}
        template_vars['job_url'] = 'http://%s%s' % (site_domain, job.get_absolute_url())
        template_vars['job_title'] = job.title
        template_vars['job_company'] = job.company
        template_vars['job_description'] = job.description
        template_vars['job_poster_email'] = job.poster_email
        job_info = {
                    'site_name': djobberbase_settings.DJOBBERBASE_SITE_NAME,
                    'job_title': job.title,
        }
        subject = djobberbase_settings.DJOBBERBASE_EDIT_POST_ADMIN_SUBJECT % job_info
        if not job.is_active():
            subject = djobberbase_settings.DJOBBERBASE_NEW_POST_ADMIN_SUBJECT % job_info
            template_vars['job_activate_url'] = 'http://%s%s' % (site_domain, job.get_activation_url())
        template_vars['job_edit_url'] = 'http://%s%s' % (site_domain, job.get_edit_url())
        template_vars['job_deactivate_url'] = 'http://%s%s' % (site_domain, job.get_deactivation_url())
        template_vars['job_poster_ip'] = getIP(request)
        template_vars['job_post_date'] = job.created_on
        d = Context(template_vars)

        from_email = djobberbase_settings.DJOBBERBASE_ADMIN_EMAIL
        to = djobberbase_settings.DJOBBERBASE_ADMIN_EMAIL
        text_content = plaintext.render(d)
        html_content = html.render(d)
        self.email = EmailMultiAlternatives(subject, text_content, from_email, [to])
        self.email.attach_alternative(html_content, "text/html")

    def run(self):
        self.email.send()

class MailPublishPendingToUser(threading.Thread):

    def __init__(self, job, request):
        threading.Thread.__init__(self)
        plaintext = get_template('djobberbase/emails/publish_pending_to_user.txt')
        html = get_template('djobberbase/emails/publish_pending_to_user.html')
        template_vars = {}
        template_vars['job_url'] = 'http://%s%s' % (site_domain, job.get_absolute_url())
        template_vars['job_title'] = job.title
        template_vars['job_company'] = job.company
        template_vars['job_description'] = job.description
        template_vars['job_poster_email'] = job.poster_email
        template_vars['job_post_date'] = job.created_on
        d = Context(template_vars)
        job_info = {
                    'site_name': djobberbase_settings.DJOBBERBASE_SITE_NAME,
                    'job_title': job.title,
        }
        subject = djobberbase_settings.DJOBBERBASE_MAIL_PENDING_SUBJECT %  job_info
        from_email = djobberbase_settings.DJOBBERBASE_ADMIN_EMAIL
        to = job.poster_email
        text_content = plaintext.render(d)
        html_content = html.render(d)
        self.email = EmailMultiAlternatives(subject, text_content, from_email, [to])
        self.email.attach_alternative(html_content, "text/html")

    def run(self):
        self.email.send()

class MailPublishToUser(threading.Thread):

    def __init__(self, job, request):
        threading.Thread.__init__(self)
        plaintext = get_template('djobberbase/emails/publish_to_user.txt')
        html = get_template('djobberbase/emails/publish_to_user.html')
        template_vars = {}
        template_vars['job_url'] = 'http://%s%s' % (site_domain, job.get_absolute_url())
        template_vars['job_edit_url'] = 'http://%s%s' % (site_domain, job.get_edit_url())
        template_vars['job_deactivate_url'] = 'http://%s%s' % (site_domain, job.get_deactivation_url())
        template_vars['site_name'] = djobberbase_settings.DJOBBERBASE_SITE_NAME
        d = Context(template_vars)
        job_info = {
                    'site_name': djobberbase_settings.DJOBBERBASE_SITE_NAME,
                    'job_title': job.title,
        }
        subject = djobberbase_settings.DJOBBERBASE_MAIL_PUBLISH_SUBJECT % job_info 
        from_email = djobberbase_settings.DJOBBERBASE_ADMIN_EMAIL
        to = job.poster_email
        text_content = plaintext.render(d)
        html_content = html.render(d)
        self.email = EmailMultiAlternatives(subject, text_content, from_email, [to])
        self.email.attach_alternative(html_content, "text/html")

    def run(self):
        self.email.send()

class MailApplyOnline(threading.Thread):

    def __init__(self, job, request):
        threading.Thread.__init__(self)
        job_info = {
                    'site_name': djobberbase_settings.DJOBBERBASE_SITE_NAME,
                    'job_title': job.title,
        }
        subject = djobberbase_settings.DJOBBERBASE_MAIL_APPLY_ONLINE_SUBJECT % job_info
        from_email = djobberbase_settings.DJOBBERBASE_ADMIN_EMAIL
        to = job.poster_email
        msg = request.POST['apply_msg']
        self.email = EmailMessage(subject, msg, from_email, [to], headers = {'Reply-To': request.POST['apply_email']})
        if 'apply_cv' in request.FILES.keys():
            name = unicode(int(time()))+'_'+request.FILES['apply_cv'].name
            handle_uploaded_file(request.FILES['apply_cv'], name)
            self.email.attach_file(djobberbase_settings.DJOBBERBASE_FILE_UPLOADS + name)
            delete_uploaded_file(djobberbase_settings.DJOBBERBASE_FILE_UPLOADS + name)

    def run(self):
        self.email.send()



# -*- coding: utf-8 -*-

from djobberbase.conf import settings
from djobberbase.models import Job, Type

def general_settings(request):
    tv = {}
    tv['DJOBBERBASE_SITE_NAME'] = settings.DJOBBERBASE_SITE_NAME
    tv['DJOBBERBASE_HTML_TITLE'] = settings.DJOBBERBASE_HTML_TITLE 
    tv['DJOBBERBASE_SITE_KEYWORDS'] = settings.DJOBBERBASE_SITE_KEYWORDS
    tv['DJOBBERBASE_SITE_DESCRIPTION'] = settings.DJOBBERBASE_SITE_DESCRIPTION
    return tv

def categories_and_types(request):
    tv = {}
    tv['djobberbase_categories'] = Category.objects.all().order_by('category_order')
    tv['djobberbase_types'] = Type.objects.all()
    



# -*- coding: utf-8 -*-

from django import template
from djobberbase.models import Job, Category, Type, JobStat
from django.utils.safestring import mark_safe
from django.db.models import Count
import re

# latest jobs template tag
def do_latest_jobs(parser, token):
    bits = token.split_contents()
    if len(bits) != 4:
        raise template.TemplateSyntaxError("'get_latest_jobs' tag takes exactly four arguments")    
    return LatestJobsNode(bits[1], bits[3])

class LatestJobsNode(template.Node):
    def __init__(self, num, varname):
        self.num = int(num)
        self.varname = varname

    def render(self, context):
        context[self.varname] = Job.active.all().order_by('-created_on')[:self.num]
        return ''

# spotlight jobs template tag
def do_spotlight_jobs(parser, token):
    bits = token.split_contents()
    if len(bits) != 4:
        raise template.TemplateSyntaxError("'get_spotlight_jobs' tag takes exactly four arguments")
    return SpotlightJobsNode(bits[1], bits[3])

class SpotlightJobsNode(template.Node):
    def __init__(self, num, varname):
        self.num = int(num)
        self.varname = varname

    def render(self, context):
        context[self.varname] = Job.active.filter(spotlight=True).order_by('-created_on')[:self.num]
        return ''

#most applied jobs template tag
def do_most_applied_jobs(parser, token):
    bits = token.split_contents()
    if len(bits) != 4:
        raise template.TemplateSyntaxError("'get_most_applied_jobs' tag takes exactly four arguments")
    return MostAppliedJobsNode(bits[1], bits[3])

class MostAppliedJobsNode(template.Node):
    def __init__(self, num, varname):
        self.num = int(num)
        self.varname = varname

    def render(self, context):
        applications = JobStat.objects.filter(stat_type='A').values('job').annotate(Count('job')).order_by('-job__count')[:self.num]
        jobs = []
        for application in applications:
            jobs.append(Job.active.get(pk=application['job']))
        context[self.varname] = jobs
        return ''


# categories template tag
def do_categories(parser, token):
    return CategoriesNode()

class CategoriesNode(template.Node):
    def render(self, context):
        context['total_jobs'] = Job.active.all().count()
        context['categories'] = Category.objects.all().order_by('category_order')
        return ''

def do_jobtypes(parser, token):
    return JobtypesNode()

class JobtypesNode(template.Node):
    def render(self, context):
        context['jobtypes'] = Type.objects.all()
        return ''

NOFOLLOW_RE = re.compile(u'<a (?![^>]*rel=["\']nofollow[\'"])' \
                         u'(?![^>]*href=["\']\.{0,2}/[^/])',
                         re.UNICODE|re.IGNORECASE)
def nofollow(content):
    return mark_safe(re.sub(NOFOLLOW_RE, u'<a rel="nofollow" ', content))

register = template.Library()
register.tag('get_latest_jobs', do_latest_jobs)
register.tag('get_spotlight_jobs', do_spotlight_jobs)
register.tag('get_most_applied_jobs', do_most_applied_jobs)
register.tag('get_categories', do_categories)
register.tag('get_jobtypes', do_jobtypes)
register.filter(nofollow)



# -*- coding: utf-8 -*-

from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Rss201rev2Feed
from django.shortcuts import get_object_or_404
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from djobberbase.models import Category, Job
from djobberbase.conf.settings import DJOBBERBASE_SITE_NAME

class LatestJobsFeed(Feed):

    def get_object(self, request, var_name):
        if var_name == 'all':
            return None
        else:
            return get_object_or_404(Category, var_name=var_name)

    def title(self, obj=None):
        t = _(' %(site_name)s RSS Job feed') % {'site_name' : DJOBBERBASE_SITE_NAME}
        if obj:
            t += _(': %(category)s jobs') % {'site_name' : obj}
        return t

    def link(self, obj=None):
        if not obj:
            return reverse('djobberbase_job_list_all')
        else:
            return obj.get_absolute_url()

    def description(self, obj=None):
        if not obj:
            return _('Latest jobs')
        else:
            return _('Latest jobs for %(category)s ') % {'category' : obj}

    def items(self, obj=None):
        jobs = Job.active.all()
        if obj:
            jobs = jobs.filter(category=obj)
        jobs = jobs.order_by('created_on')[:30]
        return jobs



# -*- coding: utf-8 -*-

import unittest
from djobberbase.models import Job, Type, Category, City
from djobberbase.conf import settings
from django.test.client import Client
from django.core.urlresolvers import reverse

class JobTestCase(unittest.TestCase):

    def setUp(self):
        ''' Set up test objects.
        '''

        # Creating a set of job categories
        self.category_1 = Category.objects.create(name='Genetic Engineering')
        self.category_2 = Category.objects.create(name='Eye Design', 
                                                            var_name='eyes')
        self.category_3 = Category.objects.create(name='Bounty Hunting', 
                                                            category_order=4)
        self.category_4 = Category.objects.create(name='Origami')

        # Creating a set of job types
        self.job_type_1 = Type.objects.create(name='Full time', 
                                                        var_name='fulltime')
        self.job_type_2 = Type.objects.create(name='Part time')
        self.job_type_3 = Type.objects.create(name='Freelance')

        # Creating a couple of cities
        self.city_1 = City.objects.create(name='Los Angeles', 
                                                        ascii_name='la')
        self.city_2 = City.objects.create(name='San Francisco')

        # Creating a job
        self.job_1 = Job.objects.create(category=self.category_1, 
                jobtype=self.job_type_1, 
                title='Genetist needed', 
                description='A new job', 
                company='Tyrell Corporation', 
                city=self.city_1, 
                poster_email='hr@tyrellcorp.com')

        # Creating a job with outside location
        self.job_2 = Job.objects.create(category=self.category_2, 
                jobtype=self.job_type_1, 
                title='WANTED: Eye Designer', 
                description='Must be able to put up with low temperatures.', 
                company='Tyrell Corp.', 
                city=None, 
                outside_location='Las Vegas', 
                poster_email='hr@tyrellcorp.com')

        # Set up a Client
        self.client = Client()

    def tearDown(self):
        ''' Tear down everything.
        '''
        self.category_1.delete()
        self.category_2.delete()
        self.category_3.delete()
        self.category_4.delete()
        self.job_type_1.delete()
        self.job_type_2.delete()
        self.job_type_3.delete()
        self.city_1.delete()
        self.city_2.delete()
        self.job_1.delete()
        self.job_2.delete()
        del self.client

    def testSlugs(self):
        # Test category slugs
        self.assertEqual(self.category_1.var_name, 'genetic-engineering')
        self.assertEqual(self.category_2.var_name, 'eyes')
        self.assertEqual(self.category_3.var_name, 'bounty-hunting')
        self.assertEqual(self.category_4.var_name, 'origami')

        # Test job type slugs
        self.assertEqual(self.job_type_1.var_name, 'fulltime')
        self.assertEqual(self.job_type_2.var_name, 'part-time')
        self.assertEqual(self.job_type_3.var_name, 'freelance')

        # Test city slugs
        self.assertEqual(self.city_1.ascii_name, 'la')
        self.assertEqual(self.city_2.ascii_name, 'san-francisco')

        # Test job slugs
        self.assertEqual(self.job_1.joburl, 
          'genetist-needed-'+settings.DJOBBERBASE_AT_URL+'-tyrell-corporation')
        self.assertEqual(self.job_2.joburl, 
             'wanted-eye-designer-'+settings.DJOBBERBASE_AT_URL+'-tyrell-corp')

        # Test company slugs
        self.assertEqual(self.job_1.company_slug, 'tyrell-corporation')
        self.assertEqual(self.job_2.company_slug, 'tyrell-corp')

    def testCategoryOrder(self):
        self.assertEqual(self.category_1.category_order, 0)
        self.assertEqual(self.category_2.category_order, 1)
        self.assertEqual(self.category_3.category_order, 4)
        self.assertEqual(self.category_4.category_order, 5)

    def testInitialJobStatus(self):
        ''' Ensure that initial job status is Job.TEMPORARY.
        '''
        self.assertEqual(self.job_1.status, Job.TEMPORARY)
        self.assertEqual(self.job_2.status, Job.TEMPORARY)

    def testActivateJob(self):
        ''' Test activation.
        '''
        self.job_1.activate()
        self.assertEqual(self.job_1.status, Job.ACTIVE)
        self.job_2.activate()
        self.assertEqual(self.job_2.status, Job.ACTIVE)

    def testDeactivateJob(self):
        ''' Test deactivation.
        '''
        self.job_1.deactivate()
        self.assertEqual(self.job_1.status, Job.INACTIVE)
        self.job_2.deactivate()
        self.assertEqual(self.job_2.status, Job.INACTIVE)

    def testApprovedEmail(self):
        ''' Activate job_1 and making sure job_2 will be automatically
            published.
        '''
        self.job_1.activate()
        self.assertEqual(self.job_2.email_published_before(), True)

    def testIndexView(self):
        response = self.client.get(reverse('djobberbase_job_list'))
        # Check that the response is 200 OK
        self.failUnlessEqual(response.status_code, 200)
        # Assert that number of categories is correct
        self.assertEqual(len(response.context['categories']), 4)

    def testActivateView(self):
        response = self.client.post(self.job_1.get_activation_url())
        # Check that the response is 200 OK
        self.failUnlessEqual(response.status_code, 200)
        # Assert that job status has been changed
        self.assertEqual(response.context['page_type'], 'activate')

    def testDeactivateView(self):
        response = self.client.get(self.job_1.get_deactivation_url())
        # Check that the response is 200 OK.
        self.failUnlessEqual(response.status_code, 200)
        # Assert that job status has been changed
        self.assertEqual(response.context['page_type'], 'deactivate')




# -*- coding: utf-8 -*-

from django import forms
from djobberbase.models import Job, Category, Type, JobStat
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from djobberbase.conf import settings as djobberbase_settings
from django.utils.translation import ugettext_lazy as _
from datetime import datetime, timedelta

class HorizRadioRenderer(forms.RadioSelect.renderer):
    """ this overrides widget method to put radio buttons horizontally
        instead of vertically.
    """
    def render(self):
        """Outputs radios"""
        return mark_safe(u'\n'.join([u'%s\n' % w for w in self]))

class JobForm(forms.ModelForm):
    class Meta:
        model = Job
        fields = ('category', 'jobtype', 'title', 'description', 'company', \
        'city', 'outside_location', 'url', 'poster_email', 'apply_online')
        widgets = {
            'jobtype': forms.RadioSelect(renderer=HorizRadioRenderer),
            'title': forms.TextInput(attrs={'size':50}),
            'description': forms.Textarea(attrs={'rows':15, 'cols':80}),
            'city': forms.Select(attrs={'id':'city_id'}),
            'outside_location': forms.TextInput(attrs={'id':'location_outside_ro_where', \
             'maxlength':140, 'size':50}),
            'company': forms.TextInput(attrs={'size':40}),
            'url': forms.TextInput(attrs={'size':35}),
            'poster_email': forms.TextInput(attrs={'size':40}),
        }

    def __init__(self, *args, **kwargs):
        super(JobForm, self).__init__(*args, **kwargs)
        self.fields['jobtype'].empty_label = None
        self.fields['category'].empty_label = None
        try:               
            self.fields['category'].initial = Category.objects.all()[0].id
        except IndexError:
            pass
        try:
            self.fields['jobtype'].initial = Type.objects.all()[0].id
        except IndexError:
            pass

class CaptchaJobForm(JobForm):
    if djobberbase_settings.DJOBBERBASE_CAPTCHA_POST == "simple":
        from captcha.fields import CaptchaField
        captcha = CaptchaField()
    else:
        pass

class ApplicationForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self.applicant_data = kwargs.pop('applicant_data')
        super(ApplicationForm, self).__init__(*args, **kwargs)
    apply_name = forms.CharField(max_length=50, widget=forms.TextInput(attrs={'id':'apply_name', 'size':30}))
    apply_email = forms.EmailField(max_length=50, widget=forms.TextInput(attrs={'id':'apply_email', 'size':30}))
    apply_msg = forms.CharField(widget=forms.Textarea(attrs={'rows':15, 'cols':60, 'id':'apply_msg'}))
    apply_cv = forms.FileField(required=False)

    if djobberbase_settings.DJOBBERBASE_CAPTCHA_APPLICATION == "simple":
        from captcha.fields import CaptchaField
        captcha = CaptchaField()     

    def clean(self):
        cleaned_data = self.cleaned_data
        ip = self.applicant_data['ip']
        mb = self.applicant_data['mb']
        previous_applications = JobStat.objects.filter(created_on__range=mb, ip=ip, stat_type='A')
        m = previous_applications.count()
        if m > 0:
            #Getting how many minutes until user can apply again
            d1 = previous_applications.latest('created_on').created_on + timedelta(minutes=djobberbase_settings.DJOBBERBASE_MINUTES_BETWEEN)
            d2 = datetime.now()
            remaining = d1 - d2
            remaining = remaining.seconds / 60
            raise forms.ValidationError(_('You need to wait %(remaining)s more minute(s) before you can apply for a job again.') % {'remaining': remaining+1})

        if cleaned_data['apply_cv']:
            #checking if cv extension is permitted
            extension = cleaned_data['apply_cv'].name.lower().split('.')[-1]
            if extension not in djobberbase_settings.DJOBBERBASE_CV_EXTENSIONS:
                raise forms.ValidationError(_('Your resume/CV has an invalid extension.'))
            #checking cv size does not exceed the permitted one
            permitted_size = djobberbase_settings.DJOBBERBASE_MAX_UPLOAD_SIZE
            if cleaned_data['apply_cv']._size > permitted_size:
                raise forms.ValidationError(_('Your resume/CV must not exceed the file size limit. (%(size)sMB)') % {'size': (permitted_size/1024)/1024})

        return cleaned_data          



# -*- coding: utf-8 -*-

from django.db import models

class TempJobsManager(models.Manager):
    def get_query_set(self):
        return super(TempJobsManager, self).get_query_set() \
                        .filter(status=self.model.TEMPORARY)

class ActiveJobsManager(models.Manager):
    def get_query_set(self):
        return super(ActiveJobsManager, self).get_query_set() \
                        .filter(status=self.model.ACTIVE)



# -*- coding: utf-8 -*-

from django.db import models
from django.template.defaultfilters import slugify
from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe
from django.utils.encoding import smart_str, force_unicode
from django.utils.translation import ugettext_lazy as _
from djobberbase.helpers import last_hour, getIP
from djobberbase.managers import ActiveJobsManager, TempJobsManager
from djobberbase.conf import settings as djobberbase_settings
import datetime
import uuid
import time
try: 
   from hashlib import md5
except ImportError:
   from md5 import md5

class Category(models.Model):
    ''' The Category model, very straight forward. Includes a get_total_jobs
        method that returns the total of jobs with that category.
        The save() method is overriden so it can automatically asign
        a category order in case no one is provided.
    '''
    name = models.CharField(_('Name'), unique=True, max_length=32, blank=False)
    var_name = models.SlugField(_('Slug'), unique=True, max_length=32, blank=False)
    title = models.TextField(_('Title'), blank=True)
    description = models.TextField(_('Description'), blank=True)
    keywords = models.TextField(_('Keywords'), blank=True)
    category_order = models.PositiveIntegerField(_('Category order'), 
                                                    unique=True, blank=True)

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def get_total_jobs(self):
        return Job.active.filter(category=self).count()

    def __unicode__(self):
        return self.name
    
    @models.permalink
    def get_absolute_url(self):
        return ('djobberbase_job_list_category', [self.var_name])

    def save(self, *args, **kwargs):
        if not self.category_order:
            try:
                self.category_order = Category.objects.\
                                    latest('category_order').category_order + 1
            except Category.DoesNotExist:
                self.category_order = 0
        if not self.var_name:
            self.var_name = slugify(self.name)
        super(Category, self).save(*args, **kwargs)


class Type(models.Model):
    ''' The Type model, nothing special, just the name and
        var_name fields. Again, the var_name is slugified by the overriden
        save() method in case it's not provided.
    '''
    name = models.CharField(_('Name'), unique=True, max_length=16, blank=False)
    var_name = models.SlugField(_('Slug'), unique=True, max_length=32, blank=False)

    class Meta:
        verbose_name = _('Type')
        verbose_name_plural = _('Types')

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.var_name:
            self.var_name = slugify(self.name)
        super(Type, self).save(*args, **kwargs)        

class City(models.Model):
    ''' A model for cities, with a get_total_jobs method to get
        the total of jobs in that city, save() method is overriden
        to slugify name to ascii_name.
    '''
    name = models.CharField(_('Name'), unique=True, max_length=50, blank=False)
    ascii_name = models.SlugField(_('ASCII Name'), unique=True, max_length=50, blank=False)

    class Meta:
        verbose_name = _('City')
        verbose_name_plural = _('Cities')

    def get_total_jobs(self):
        return Job.active.filter(city=self).count()

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.ascii_name:
            self.ascii_name = slugify(self.name)
        super(City, self).save(*args, **kwargs) 

class Job(models.Model):
    ''' The basic job model.
    '''
    INACTIVE = 0
    TEMPORARY = 1
    ACTIVE = 2
    JOB_STATUS_CHOICES = (
        (INACTIVE, _('Inactive')),
        (TEMPORARY, _('Temporary')),
        (ACTIVE, _('Active'))
    )
    category = models.ForeignKey(Category, verbose_name=_('Category'), blank=False, null=False)
    jobtype = models.ForeignKey(Type, verbose_name=_('Job Type'), blank=False, null=False)
    title = models.CharField(verbose_name=_('Title'), max_length=100, blank=False)
    description = models.TextField(_('Description'), blank=False)
    description_html = models.TextField(editable=False)
    company = models.CharField(_('Company'), max_length=150, blank=False)
    company_slug = models.SlugField(max_length=150, 
                                            blank=False, editable=False)
    city = models.ForeignKey(City, verbose_name=_('City'), null=True, blank=True)
    outside_location = models.CharField(_('Outside location'), max_length=150, blank=True)
    #url of the company
    url = models.URLField(verify_exists=False, max_length=150, blank=True)    
    created_on = models.DateTimeField(_('Created on'), editable=False, \
                                        default=datetime.datetime.now())
    status = models.IntegerField(choices=JOB_STATUS_CHOICES, default=TEMPORARY)
    views_count = models.IntegerField(editable=False, default=0)
    auth = models.CharField(blank=True, editable=False, max_length=32)
    #url of the job post
    joburl = models.CharField(blank=True, editable=False, max_length=32)
    poster_email = models.EmailField(_('Poster email'), blank=False, help_text=_('Applications will be sent to this address.'))
    apply_online = models.BooleanField(default=True, verbose_name=_('Allow online applications.'),
                                    help_text=_('If you are unchecking this, then add a description on how to apply online!'))
    spotlight = models.BooleanField(_('Spotlight'), default=False)    
    objects = models.Manager()
    active = ActiveJobsManager()
    temporary = TempJobsManager()

    class Meta:
        verbose_name = _('Job')
        verbose_name_plural = _('Jobs')

    def __unicode__(self):
        return self.title

    def get_application_count(self):
        return JobStat.objects.filter(job=self, stat_type='A').count()
        
    def increment_view_count(self, request):
        lh=last_hour()
        ip=getIP(request)
        hits=JobStat.objects.filter(created_on__range=lh, 
                                        ip=ip, stat_type='H', job=self).count()
        if hits < djobberbase_settings.DJOBBERBASE_MAX_VISITS_PER_HOUR:
            self.views_count=self.views_count+1
            self.save()
            new_hit = JobStat(ip=ip, stat_type='H', job=self)
            new_hit.save()

    def is_active(self):
        return self.status == self.ACTIVE

    def is_temporary(self):
        return self.status == self.TEMPORARY

    def get_status_with_icon(self):
        from settings import ADMIN_MEDIA_PREFIX
        icon = '<img src="%(admin_media)simg/admin/%(image)s" \
                alt="%(status)s" /> %(status)s'
        image = {
            self.ACTIVE: 'icon-yes.gif',
            self.TEMPORARY: 'icon-unknown.gif',
            self.INACTIVE: 'icon-no.gif',
        }[self.status]
        return icon % {'admin_media':ADMIN_MEDIA_PREFIX,
                       'image': image,
                       'status': self.JOB_STATUS_CHOICES[self.status][1]}
    get_status_with_icon.allow_tags = True
    get_status_with_icon.admin_order_field = 'status'
    get_status_with_icon.short_description = 'Status'

    def activate(self):
        self.status = self.ACTIVE
        self.save()

    def deactivate(self):
        self.status = self.INACTIVE
        self.save()

    def email_published_before(self):
        return Job.active.exclude(pk=self.id) \
                          .filter(poster_email=self.poster_email).count() > 0

    @models.permalink
    def get_edit_url(self):
        return ('djobberbase_job_edit', [self.id, self.auth])

    @models.permalink
    def get_absolute_url(self):
        return ('djobberbase_job_detail', [self.id, self.joburl])

    @models.permalink
    def get_activation_url(self):
        return ('djobberbase_job_activate', [self.id, self.auth])		

    @models.permalink
    def get_deactivation_url(self):
        return ('djobberbase_job_deactivate', [self.id, self.auth])

    def clean(self):
        #making sure a job location is selected/typed
        if self.city:
            self.outside_location = ''
        elif len(self.outside_location.strip()) > 0:
            self.city = None
        else:
            raise ValidationError(_('You must select or type a job location.'))

    def save(self, *args, **kwargs):
        #saving auth code
        if not self.auth:
            self.auth = md5(unicode(self.id) + \
                            unicode(uuid.uuid1()) + \
                            unicode(time.time()) ).hexdigest()
        #saving company slug
        self.company_slug = slugify(self.company)

        #saving job url
        self.joburl = slugify(self.title) + \
                        '-' + djobberbase_settings.DJOBBERBASE_AT_URL + \
                        '-' + slugify(self.company)

        #saving with textile
        if djobberbase_settings.DJOBBERBASE_MARKUP_LANGUAGE == 'textile':
            import textile
            self.description_html = mark_safe(
                                        force_unicode(
                                            textile.textile(
                                                smart_str(self.description))))
        #or markdown
        elif djobberbase_settings.DJOBBERBASE_MARKUP_LANGUAGE == 'markdown':
            import markdown
            self.description_html = mark_safe(
                                        force_unicode(
                                            markdown.markdown(
                                                smart_str(self.description))))
        else:
            self.description_html = self.description
        super(Job, self).save(*args, **kwargs)
        
class JobStat(models.Model):
    APPLICATION = 'A'
    HIT = 'H'
    SPAM = 'S'
    STAT_TYPES = (
        (APPLICATION, _('Application')),
        (HIT, _('Hit')),
        (SPAM, _('Spam')),
    )
    job = models.ForeignKey(Job)
    created_on = models.DateTimeField(default=datetime.datetime.now())
    ip = models.IPAddressField()
    stat_type = models.CharField(max_length=1, choices=STAT_TYPES)

    class Meta:
        verbose_name = _('Job Stat')
        verbose_name_plural = _('Job Stats')
    
    def __unicode__(self):
        if self.stat_type == 'A':
            u = 'Job application for "' +self.job.title+ '" from IP: '+ self.ip
        elif self.stat_type == 'H':
            u = 'Visit for "' +self.job.title+ '" from IP: ' +self.ip
        elif self.stat_type == 'S':
            u = 'Spam report for "' +self.job.title+ '" from IP: ' + self.ip
        return u
        
class JobSearch(models.Model):
    keywords = models.CharField(_('Keywords'), max_length=100, blank=False)
    created_on = models.DateTimeField(_('Created on'), default=datetime.datetime.now())
    
    class Meta:
        verbose_name = _('Search')
        verbose_name_plural = _('Searches')
    
    def __unicode__(self):
        return self.keywords



# -*- coding: utf-8 -*-

from django.shortcuts import get_object_or_404, redirect, render_to_response
from djobberbase.models import Job, Category, Type, JobStat, JobSearch, City
from djobberbase.postman import *
from django.views.generic.list_detail import object_detail, object_list
from django.views.generic.create_update import create_object, update_object
from django.core.context_processors import csrf
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django.template import RequestContext
from djobberbase.helpers import *
from djobberbase.forms import ApplicationForm, JobForm
from djobberbase.conf import settings as djobberbase_settings
from django.db.models import Count
from django.http import Http404

def job_detail(request, job_id, joburl):
    ''' Displays an active job and its application form depending if 
        the job has online applications or not. Handles the job applications
        and sends notifications emails.
    '''
    try:        
        job = Job.active.get(pk=job_id, joburl=joburl)
        extra_context = {'page_type':'detail', 
               'cv_extensions': djobberbase_settings.DJOBBERBASE_CV_EXTENSIONS,
               'markup_lang': djobberbase_settings.DJOBBERBASE_MARKUP_LANGUAGE}

        # Increment views
        job.increment_view_count(request)

        # Gets poster ip
        ip = getIP(request)

        # Only if the job has online applications ON and application 
        # notifications are activated can the user apply online
        mb = minutes_between()
        if job.apply_online and djobberbase_settings.\
            DJOBBERBASE_APPLICATION_NOTIFICATIONS:

            # Add CSRF protection
            extra_context.update(csrf(request))

            # If it's a job application
            if request.method == 'POST':

                # Gets the application
                form = ApplicationForm(request.POST, 
                                       request.FILES, 
                                       applicant_data={'ip':ip, 'mb':mb})

                # If the form is OK then send it to the job poster
                if form.is_valid():
                    application_mail = MailApplyOnline(job, request)
                    application_mail.start()

                    #Save JobStat application
                    ja = JobStat(job=job, ip=ip, stat_type='A')
                    ja.save()
                    messages.add_message(request, 
                                messages.INFO, 
                                _('Your application was sent successfully.'))
                    extra_context['page_type'] = 'application'
                    queryset = Job.active.filter(joburl=joburl)
                    return object_detail(request, queryset=queryset,
                                        object_id=job_id,
                                        extra_context=extra_context)
                else:
                    extra_context['form_error'] = True 

            # Else create an empty application form
            else:
                form = ApplicationForm(applicant_data={'ip':ip, 'mb':mb})
            extra_context['apform'] = form
            extra_context['object'] = job
            return render_to_response('djobberbase/job_detail.html',
                                       extra_context,
                                       context_instance=RequestContext(request))

        # Only display the job, without an application form
        else:
            queryset = Job.active.filter(joburl=joburl)
            return object_detail(request, queryset=queryset,
                                object_id=job_id,
                                extra_context=extra_context)

    # Instead of throwing a 404 error redirect to job unavailable page 
    except Job.DoesNotExist:
        return redirect('djobberbase_job_unavailable', permanent=True)

def job_verify(request, job_id, auth):
    ''' A view to display a newly created job.
    '''
    queryset = Job.objects.filter(auth=auth)
    # Setting page_type as 'verify' in order to 
    # show edit and cancelation buttons in the template
    extra_context = {'page_type':'verify', 
               'markup_lang': djobberbase_settings.DJOBBERBASE_MARKUP_LANGUAGE}
    return object_detail(request, queryset=queryset, 
                            object_id=job_id, extra_context=extra_context)

def jobs_category(request, cvar_name=None, tvar_name=None):
    ''' Displays a job list by category and/or job type but
        those two are optional.
    '''
    extra_context = {}
    queryset = Job.active.all()
    if cvar_name:
        category = get_object_or_404(Category, var_name=cvar_name)
        queryset = queryset.filter(category=category)
        extra_context['selected_category'] = category
    if tvar_name:
        jobtype = get_object_or_404(Type, var_name=tvar_name)
        queryset = queryset.filter(jobtype=jobtype)
        extra_context['selected_jobtype'] = jobtype
    return object_list(request, queryset=queryset,
                    extra_context=extra_context,
                    paginate_by=djobberbase_settings.DJOBBERBASE_JOBS_PER_PAGE)

def jobs_in_city(request, city_name, tvar_name=None):
    ''' Display a job list by city and job type (optional).
    '''
    city = get_object_or_404(City, ascii_name=city_name)
    queryset = Job.active.filter(city=city)
    extra_context = {'city': city}
    if tvar_name:
        jobtype = get_object_or_404(Type, var_name=tvar_name)
        queryset = queryset.filter(jobtype=jobtype)
        extra_context['selected_jobtype'] = jobtype                     
    return object_list(request, queryset=queryset,
                    extra_context=extra_context,
                    paginate_by=djobberbase_settings.DJOBBERBASE_JOBS_PER_PAGE)

def jobs_in_other_cities(request):
    ''' Displays a list with jobs in cities outside.
    '''
    queryset = Job.active.filter(city=None)
    return object_list(request, queryset=queryset)

def companies(request):
    ''' Displays the companies that have active jobs
        posted on the site.
    '''
    queryset = Job.active.values('company', 'company_slug') \
               .annotate(Count('company'))
    return object_list(request, queryset=queryset,
                                template_name='djobberbase/company_list.html')

def jobs_at(request, company_slug, tvar_name=None):
    ''' Displays a job list by company, jobtype is optional.
    '''
    queryset = Job.active.filter(company_slug=company_slug)
    if tvar_name:
        jobtype = get_object_or_404(Type, var_name=tvar_name)
        queryset = queryset.filter(jobtype=jobtype)
        extra_context['selected_jobtype'] = jobtype    
    return object_list(request, queryset=queryset)

def job_confirm(request, job_id, auth):
    ''' A view to confirm a recently created job, if it has been published
        by a previously approved user then it gets automatically published,
        if not then it will need to be verified by a moderator.
    '''
    job = get_object_or_404(Job, pk=job_id, auth=auth)
    if job.status not in (Job.ACTIVE, Job.TEMPORARY):
        raise Http404
    new_post = job.is_temporary()
    requires_mod = not job.email_published_before() and \
                 djobberbase_settings.DJOBBERBASE_ENABLE_NEW_POST_MODERATION
    if requires_mod:
        messages.add_message(request, 
                       messages.INFO, 
                       _('Your job post needs to be verified by a moderator.'))
        if djobberbase_settings.DJOBBERBASE_POSTER_NOTIFICATIONS:
            pending_email = MailPublishPendingToUser(job, request)
            pending_email.start()
    else:
        messages.add_message(request, 
                             messages.INFO, 
                             _('Your job post has been published.'))
        if not job.is_active():
            job.activate()
        if new_post:
            if djobberbase_settings.DJOBBERBASE_POSTER_NOTIFICATIONS:
                publish_email = MailPublishToUser(job, request)
                publish_email.start()    
    queryset = Job.objects.all()
    if djobberbase_settings.DJOBBERBASE_ADMIN_NOTIFICATIONS:
        admin_email = MailPublishToAdmin(job, request)
        admin_email.start()
    return object_detail(request, queryset=queryset,
                         object_id=job_id,
                         extra_context={'page_type':'confirm'})
                         
def job_edit(request, job_id, auth):
    ''' A view for editing published or unpublished job posts.
    '''
    job = get_object_or_404(Job, pk=job_id, auth=auth)
    if job.status not in (Job.ACTIVE, Job.TEMPORARY):
        raise Http404
    return update_object(request, form_class=JobForm, object_id=job_id,
           post_save_redirect='../../../'+
           djobberbase_settings.DJOBBERBASE_VERIFY_URL+'/%(id)d/%(auth)s/')

def job_activate(request, job_id, auth):
    ''' Gets a job and activates it, only if it's not already activated,
        it also sends the notification mail to the poster.
    '''
    job = get_object_or_404(Job, pk=job_id, auth=auth)
    extra_context={}
    if not job.is_active():
        job.activate()
        if djobberbase_settings.DJOBBERBASE_POSTER_NOTIFICATIONS:
            publish_email = MailPublishToUser(job, request)
            publish_email.start()
        messages.add_message(request, 
                             messages.INFO, 
                             _('Your job has been activated.'))
        extra_context['page_type'] = 'activate'
    queryset=Job.active.all()
    return object_detail(request, queryset=queryset, 
                            object_id=job_id, extra_context=extra_context)

def job_deactivate(request, job_id, auth):
    ''' Deactivates a job and shows an active jobs list.    
    '''
    job = get_object_or_404(Job, pk=job_id, auth=auth)
    extra_context={}
    if job.is_active() or job.is_temporary():
        job.deactivate()
        messages.add_message(request, 
                             messages.INFO, 
                             _('Your job has been deactivated.'))
        extra_context['page_type'] = 'deactivate'
    queryset=Job.active.all()
    return object_list(request, queryset=queryset,
                    extra_context=extra_context,
                    paginate_by=djobberbase_settings.DJOBBERBASE_JOBS_PER_PAGE)

def job_search(request):
    ''' A search view, does the job but not great. Job searches should be
        handled by a proper search app, namely django-haystack.
    '''
    query_string = ''
    found_entries = Job.objects.none()
    extra_context = {'keywords': ' '}
    if ('keywords' in request.POST) and request.POST['keywords'].strip():
        request.session['keywords'] = request.POST['keywords']
        query_string = request.session['keywords']
        extra_context['keywords'] = query_string
        search_fields = ['title', 'description', 'category',
                             'jobtype', 'city', 'outside_location', 'company',]
        entry_query = get_query(query_string, search_fields)
        jobs_per_search = djobberbase_settings.DJOBBERBASE_JOBS_PER_SEARCH
        found_entries = Job.objects.filter(entry_query)\
                                     .order_by('-created_on')[:jobs_per_search]
        search = JobSearch(keywords=query_string)
        search.save()
    return object_list(request, queryset=found_entries,
                    extra_context=extra_context,
                    paginate_by=djobberbase_settings.DJOBBERBASE_JOBS_PER_PAGE)



# -*- coding: utf-8 -*-

from distutils.core import setup
setup(
    name = "django-djobberbase",
    packages = [
            "djobberbase",
            "djobberbase.conf",
            "djobberbase.templatetags",
    ],
    package_data = {       
        'djobberbase': [
            'templates/djobberbase/emails/*.txt',
            'templates/djobberbase/emails/*.html',
            'templates/djobberbase/*.html',
            'locale/es/LC_MESSAGES/django.mo',
            'locale/es/LC_MESSAGES/django.po',
            'media/css/*.css',
            'media/img/*.gif',
            'media/img/*.png',
            'media/js/*.js'
        ],
    },
    version="0.1.0",
    license="MIT",
    description = "A clone of the jobberBase job board platform written using the Django framework.",
    author = "Walter Treviño",
    author_email = "wtrevino@derelict.mx",
    url = "https://github.com/wtrevino/django-djobberbase",
    keywords = ["django", "python", "jobberbase", "job board"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)




# -*- coding: utf-8 -*-

from djobberbase.models import Category, Type, Job, City, JobStat, JobSearch
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

def activate_jobs(modeladmin, request, queryset):
    queryset.update(status=Job.ACTIVE)
activate_jobs.short_description = _('Activate selected jobs.')

def deactivate_jobs(modeladmin, request, queryset):
    queryset.update(status=Job.INACTIVE)
deactivate_jobs.short_description = _('Deactivate selected jobs.')

def mark_spotlight(modeladmin, request, queryset):
    queryset.update(spotlight=True)
mark_spotlight.short_description = _('Mark selected jobs as spotlight.')

class JobAdmin(admin.ModelAdmin):
    fieldsets = [
        (_('Job Details'), {'fields': ['jobtype', 'category', 'title', \
                                    'city', 'outside_location', 'description']}),
        (_('Company Info'), {'fields': ['company', 'url', 'poster_email']}),
        (_('Admin Info'),  {'fields': ['apply_online', 'status', 'spotlight']})
    ]
    list_display = ('title', 'company' ,'created_on', 'get_status_with_icon', 'spotlight')
    actions = [activate_jobs, deactivate_jobs, mark_spotlight]

class CategoryAdmin(admin.ModelAdmin):
    prepopulated_fields = {'var_name': ('name',)}
    list_display = ('name', 'category_order')

class TypeAdmin(admin.ModelAdmin):
    prepopulated_fields = {'var_name': ('name',)}

class CityAdmin(admin.ModelAdmin):
    prepopulated_fields = {'ascii_name': ('name',)}

class JobStatAdmin(admin.ModelAdmin):
    readonly_fields = ['job', 'created_on', 'ip', 'stat_type']

class JobSearchAdmin(admin.ModelAdmin):
    readonly_fields = ['keywords', 'created_on']    
    
admin.site.register(Category, CategoryAdmin)
admin.site.register(Type, TypeAdmin)
admin.site.register(Job, JobAdmin)
admin.site.register(City, CityAdmin)
admin.site.register(JobStat, JobStatAdmin)
admin.site.register(JobSearch, JobSearchAdmin)



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *
from djobberbase.models import Job, Category, Type, City
from djobberbase.conf import settings as djobberbase_settings
from djobberbase.feeds import LatestJobsFeed

if djobberbase_settings.DJOBBERBASE_CAPTCHA_POST == 'simple':
    from djobberbase.forms import CaptchaJobForm
    form_class = CaptchaJobForm
else:
    from djobberbase.forms import JobForm
    form_class = JobForm


urlpatterns = patterns('django.views.generic',

    #An index view
    url(r'^$',
        'list_detail.object_list', 
        { 'queryset': Job.active.all(),          
          'extra_context': {'page_type': 'index'},
          'paginate_by': djobberbase_settings.DJOBBERBASE_JOBS_PER_PAGE},
        name='djobberbase_job_list'),
    
    #Cities view    
    url(r'^'+djobberbase_settings.DJOBBERBASE_CITIES_URL+'/$',
        'list_detail.object_list', 
        { 'queryset': City.objects.all(),
          'extra_context': {'page_type': 'cities', 
                    'other_cities_total': Job.active.filter(city=None).count}},
        name='djobberbase_cities_list'),

    #post new job
    url(r'^'+djobberbase_settings.DJOBBERBASE_POST_URL+'/$',
        'create_update.create_object', 
        { 'form_class': form_class, 
          'post_save_redirect': '../'+
          djobberbase_settings.DJOBBERBASE_VERIFY_URL+'/%(id)d/%(auth)s/'},
        name='djobberbase_job_post'),

    #job unavailable
    url(r'^'+djobberbase_settings.DJOBBERBASE_UNAVAILABLE_URL+'/$',
        'simple.direct_to_template', 
        {'template': 'djobberbase/unavailable.html'},
        name='djobberbase_job_unavailable'),
)

urlpatterns += patterns('',

    #verify job
    url(r'^'+djobberbase_settings.DJOBBERBASE_VERIFY_URL+
        '/(?P<job_id>\d+)/(?P<auth>[-\w]+)/$',
        'djobberbase.views.job_verify', 
        name='djobberbase_job_verify'),

    #all jobs
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_URL+'/$',
        'djobberbase.views.jobs_category',
        name='djobberbase_job_list_all'),

    #all jobs with category
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_URL+
        '/(?P<cvar_name>[-\w]+)/$',
        'djobberbase.views.jobs_category',
        name='djobberbase_job_list_category'),

    #all jobs with category and job type    
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_URL+
        '/(?P<cvar_name>[-\w]+)/(?P<tvar_name>[-\w]+)/$',
        'djobberbase.views.jobs_category',
        name='djobberbase_job_list_category_type'),

    #Job detail    
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOB_URL+
        '/(?P<job_id>\d+)/(?P<joburl>[-\w]+)/$',
        'djobberbase.views.job_detail',
        name='djobberbase_job_detail'),
        
    #Jobs in city view
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_IN_URL+
        '/(?P<city_name>[-\w]+)/$',
        'djobberbase.views.jobs_in_city',
        name='djobberbase_jobs_in_city'),

    #Jobs in other cities
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_IN_OTHER_CITIES+'/$',
        'djobberbase.views.jobs_in_other_cities',
        name='djobberbase_jobs_in_other_cities'),

    #Jobs in city+jobtype view
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_IN_URL+
        '/(?P<city_name>[-\w]+)/(?P<tvar_name>[-\w]+)/$',
        'djobberbase.views.jobs_in_city',
        name='djobberbase_jobs_in_city_jobtype'),

    #Companies
    url(r'^'+djobberbase_settings.DJOBBERBASE_COMPANIES_URL+'/$',
        'djobberbase.views.companies',
        name='djobberbase_companies'),

    #Jobs at (company)
    url(r'^'+djobberbase_settings.DJOBBERBASE_JOBS_AT_URL+
        '/(?P<company_slug>[-\w]+)/$',
        'djobberbase.views.jobs_at',
        name='djobberbase_jobs_at'),

    #Job confirm
    url(r'^'+djobberbase_settings.DJOBBERBASE_CONFIRM_URL+
        '/(?P<job_id>\d+)/(?P<auth>[-\w]+)/$',
        'djobberbase.views.job_confirm',
        name='djobberbase_job_confirm'),

    #Edit job
    url(r'^'+djobberbase_settings.DJOBBERBASE_POST_URL+
        '/(?P<job_id>\d+)/(?P<auth>[-\w]+)/$',
        'djobberbase.views.job_edit',
        name='djobberbase_job_edit'),

    #Activate job
    url(r'^'+djobberbase_settings.DJOBBERBASE_ACTIVATE_URL+
        '/(?P<job_id>\d+)/(?P<auth>[-\w]+)/$',
        'djobberbase.views.job_activate',
        name='djobberbase_job_activate'),

    #Deactivate job
    url(r'^'+djobberbase_settings.DJOBBERBASE_DEACTIVATE_URL+
        '/(?P<job_id>\d+)/(?P<auth>[-\w]+)/$',
        'djobberbase.views.job_deactivate',
        name='djobberbase_job_deactivate'),

    #Search
    url(r'^'+djobberbase_settings.DJOBBERBASE_SEARCH_URL+'/$',
        'djobberbase.views.job_search',
        name='djobberbase_job_search'),

    #Feed
    url(r'^rss/(?P<var_name>[-\w]+)/$',
        LatestJobsFeed(),
        name='djobberbase_feed'),

)



# -*- coding: utf-8 -*-

#                        CREDITS 
#  Based on: http://www.djangosnippets.org/snippets/73/
#
#  Modified by Sean Reifschneider to be smarter about surrounding page
#  link context.  For usage documentation see:
#
#     http://www.tummy.com/Community/Articles/django-pagination/

from django import template

register = template.Library()

def paginator(context, adjacent_pages=2):
    """
    To be used in conjunction with the object_list generic view.

    Adds pagination context variables for use in displaying first, adjacent and
    last page links in addition to those created by the object_list generic
    view.

    """
    startPage = max(context['page'] - adjacent_pages, 1)
    if startPage <= 3: startPage = 1
    endPage = context['page'] + adjacent_pages + 1
    if endPage >= context['pages'] - 1: endPage = context['pages'] + 1
    page_numbers = [n for n in range(startPage, endPage) \
            if n > 0 and n <= context['pages']]
    page_obj = context['page_obj']
    paginator = context['paginator']

    return {
        'page_obj': page_obj,
        'paginator': paginator,
        'hits': context['hits'],
        'results_per_page': context['results_per_page'],
        'page': context['page'],
        'pages': context['pages'],
        'page_numbers': page_numbers,
        'next': context['next'],
        'previous': context['previous'],
        'has_next': context['has_next'],
        'has_previous': context['has_previous'],
        'show_first': 1 not in page_numbers,
        'show_last': context['pages'] not in page_numbers,
    }

register.inclusion_tag('djobberbase/paginator.html', takes_context=True)(paginator)
